@@ -8,7 +8,7 @@
 # Copyright 2001,2009 Business Management Systems, Inc.
 # This code is under the GNU General Public License.  See COPYING for details.
 
-__version__ = '0.9.7'
+__version__ = '0.9.8'
 
 import os
 import re
@@ -21,12 +21,6 @@ from functools import wraps
 _seq_lock = thread.allocate_lock()
 _seq = 0
 
-## @fn set_flags(flags)
-# @brief Enable optional %milter actions.
-# Certain %milter actions need to be enabled before calling milter.runmilter()
-# or they throw an exception. 
-# @param flags Bit ored mask of optional actions to enable
-
 def uniqueID():
   """Return a unique sequence number (incremented on each call).
   """
@@ -227,7 +221,7 @@ class Base(object):
     # Some optional actions may be disabled by calling milter.set_flags(), or
     # by overriding the negotiate callback.  The bits include:
     # <code>ADDHDRS,CHGBODY,MODBODY,ADDRCPT,ADDRCPT_PAR,DELRCPT
-    #  CHGHDRS,QUARANTINE,CHGFROM,SETSMLIST</code>.
+    #  CHGHDRS,QUARANTINE,CHGFROM,SETSYMLIST</code>.
     # The <code>Milter.CURR_ACTS</code> bitmask is all actions
     # known when the milter module was compiled.
     # Application code can also inspect this field to determine
@@ -416,12 +410,21 @@ class Base(object):
 
   ## Set the SMTP reply code and message.
   # If the MTA does not support setmlreply, then only the
-  # first msg line is used.  Any '%' in a message line
+  # first msg line is used.  Any '%%' in a message line
   # must be doubled, or libmilter will silently ignore the setreply.
   # Beginning with 0.9.6, we test for that case and throw ValueError to avoid
   # head scratching.  What will <i>really</i> irritate you, however,
-  # is that if you carefully double any '%', your message will be
-  # sent - but with the '%' still doubled!
+  # is that if you carefully double any '%%', your message will be
+  # sent - but with the '%%' still doubled!
+  # See <a href="https://www.milter.org/developers/api/smfi_setreply">
+  # smfi_setreply</a> for more information.
+  # @param rcode The three-digit (RFC 821/2821) SMTP reply code as a string. 
+  # rcode cannot be None, and <b>must be a valid 4XX or 5XX reply code</b>.
+  # @param xcode The extended (RFC 1893/2034) reply code. If xcode is None,
+  # no extended code is used. Otherwise, xcode must conform to RFC 1893/2034.
+  # @param msg The text part of the SMTP reply. If msg is None,
+  # an empty message is used.
+  # @param ml  Optional additional message lines.
   def setreply(self,rcode,xcode=None,msg=None,*ml):
     for m in (msg,)+ml:
       if 1 in [len(s)&1 for s in R.findall(m)]:
@@ -429,17 +432,29 @@ class Base(object):
     return self._ctx.setreply(rcode,xcode,msg,*ml)
 
   ## Tell the MTA which macro names will be used.
-  # The <code>Milter.SETSMLIST</code> action flag must be set.
+  # This information can reduce the size of messages received from sendmail,
+  # and hence could reduce bandwidth between sendmail and your milter where
+  # that is a factor.  The <code>Milter.SETSYMLIST</code> action flag must be
+  # set.  The protocol stages are M_CONNECT, M_HELO, M_ENVFROM, M_ENVRCPT,
+  # M_DATA, M_EOM, M_EOH.
   #
   # May only be called from negotiate callback.
-  # @since 0.9.2
-  # @param stage the protocol stage to set to macro list for
-  # @param macros a string with a space delimited list of macros
-  def setsmlist(self,stage,macros):
-    if not self._actions & SETSMLIST: raise DisabledAction("SETSMLIST")
-    if type(macros) in (list,tuple):
-      macros = ' '.join(macros)
-    return self._ctx.setsmlist(stage,macros)
+  # @since 0.9.8, previous version was misspelled!
+  # @param stage the protocol stage to set to macro list for, 
+  # one of the M_* constants defined in Milter
+  # @param macros space separated and/or lists of strings
+  def setsymlist(self,stage,*macros):
+    if not self._actions & SETSYMLIST: raise DisabledAction("SETSYMLIST")
+    a = []
+    for m in macros:
+      try:
+        m = m.encode('utf8')
+      except: pass
+      try:
+        m = m.split(' ')
+      except: pass
+      a += m
+    return self._ctx.setsmlist(stage,' '.join(a))
 
   # Milter methods which can only be called from eom callback.
 
@@ -703,28 +718,7 @@ def envcallback(c,args):
 # @param socketname the socket to be passed to milter.setconn()
 # @param timeout the time in secs the MTA should wait for a response before 
 #	considering this %milter dead
-def runmilter(name,socketname,timeout = 0):
-  # This bit is here on the assumption that you will be starting this filter
-  # before sendmail.  If sendmail is not running and the socket already exists,
-  # libmilter will throw a warning.  If sendmail is running, this is still
-  # safe if there are no messages currently being processed.  It's safer to
-  # shutdown sendmail, kill the filter process, restart the filter, and then
-  # restart sendmail.
-  pos = socketname.find(':')
-  if pos > 1:
-    s = socketname[:pos]
-    fname = socketname[pos+1:]
-  else:
-    s = "unix"
-    fname = socketname
-  if s == "unix" or s == "local":
-    print "Removing %s" % fname
-    try:
-      os.unlink(fname)
-    except os.error, x:
-      import errno
-      if x.errno != errno.ENOENT:
-        raise milter.error(x)
+def runmilter(name,socketname,timeout = 0,rmsock=True):
 
   # The default flags set include everything
   # milter.set_flags(milter.ADDHDRS)
@@ -755,6 +749,14 @@ def runmilter(name,socketname,timeout = 0):
         unknown=lambda ctx,cmd: ctx.getpriv().unknown(cmd),
         negotiate=ncb
   )
+
+  # We remove the socket here by default on the assumption that you will be
+  # starting this filter before sendmail.  If sendmail is not running and the
+  # socket already exists, libmilter will throw a warning.  If sendmail is
+  # running, this is still safe if there are no messages currently being
+  # processed.  It's safer to shutdown sendmail, kill the filter process,
+  # restart the filter, and then restart sendmail.
+  milter.opensocket(rmsock)
   start_seq = _seq
   try:
     milter.main()
